/*
    Copyright (C) 2019 - 2024 Remon Sijrier

    This file is part of Traverso

    Traverso is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
*/

#ifndef T_GAIN_GROUP_COMMAND_H
#define T_GAIN_GROUP_COMMAND_H

#include "TCommand.h"
#include <QPoint>

class GainCommand;
class TAudioProcessingNode;

class TGainGroupCommand : public TCommand
{
    Q_OBJECT

public :
    TGainGroupCommand(TContextItem* context);
    ~TGainGroupCommand();

    int begin_hold();
    int finish_hold();
    void cancel_action();
    void process_collected_number(const QString & collected);
    void set_cursor_shape(int useX, int useY);

    int jog();

    bool is_hold_command() const {return true;}
    int prepare_actions();
    int do_action();
    int undo_action();

    bool wants_cursor_position_to_be_restored() const {return true;}

    void add_audio_processing_node(TAudioProcessingNode *audioProcessingNode, const QVariantList& args);

private:
    std::vector<std::unique_ptr<GainCommand> >	m_gainCommands;
    QPointF         m_origPos;
    TContextItem*    m_contextItem;
    bool            m_primaryGainOnly;

    QString get_db_string_from_object();


public slots:
    void increase_gain();
    void decrease_gain();
    void reset_gain();
    void toggle_primary_gain_only();
    void numerical_input();
};

#endif



