/*
Copyright (C) 2005-2008 Remon Sijrier 

This file is part of Traverso

Traverso is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.

*/

#ifndef TAUDIOCLIPMANAGER_H
#define TAUDIOCLIPMANAGER_H

#include "TContextItem.h"
#include "TTimeRef.h"

#include <QDomNode>
#include <QList>

class TAudioClip;
class TSheet;

class TAudioClipManager : public TContextItem
{
	Q_OBJECT

public:
    TAudioClipManager(TSheet* sheet);
    ~TAudioClipManager();

	QDomNode get_state(QDomDocument doc, bool istemplate=false);
	int set_state( const QDomNode & node );

	
	bool is_clip_in_selection(TAudioClip* clip);
	
	void get_selected_clips(QList<TAudioClip*> & list);

	QList<TAudioClip* > get_clip_list() const;

    TTimeRef get_last_location() const;

private:
	QList<TAudioClip* >		m_clips;
	QList<TAudioClip* >		m_clipselection;
	TSheet*				m_sheet;
	
    TTimeRef 			m_lastLocation;

public slots:
	void add_clip(TAudioClip* clip);
	void remove_clip(TAudioClip* clip);
	void select_clip(TAudioClip* clip);
	void toggle_selected(TAudioClip* clip);
	
	void remove_from_selection(TAudioClip* clip);
	void add_to_selection(TAudioClip* clip);
	void update_last_frame();

	TCommand* select_all_clips();
	TCommand* invert_clip_selection();
};

#endif

//eof
